#nullable enable
using Newtonsoft.Json;

namespace FitConnect.Models.Api.Set;

public interface IReplyChannelType {
}

public class ReplyChannel {
    /// <summary>
    /// Akkreditierte Anbieter siehe
    /// https://www.bsi.bund.de/DE/Themen/Oeffentliche-Verwaltung/Moderner-Staat/De-Mail/Akkreditierte-DMDA/akkreditierte-dmda_node.html
    /// </summary>
    [JsonProperty("deMail", NullValueHandling = NullValueHandling.Ignore)]
    public DeMail? DeMail { get; set; }

    /// <summary>
    /// Siehe https://www.elster.de/elsterweb/infoseite/elstertransfer_hilfe_schnittstellen
    /// </summary>
    [JsonProperty("elster", NullValueHandling = NullValueHandling.Ignore)]
    public Elster? Elster { get; set; }

    [JsonProperty("eMail", NullValueHandling = NullValueHandling.Ignore)]
    public EMail? EMail { get; set; }

    /// <summary>
    /// Postfachadresse in einem interoperablen Servicekonto (FINK.PFISK)
    /// </summary>
    [JsonProperty("fink", NullValueHandling = NullValueHandling.Ignore)]
    public Fink? Fink { get; set; }

    public static ReplyChannel? FromType(IReplyChannelType replyChannel) =>
        replyChannel switch {
            DeMail deMail => new ReplyChannel() { DeMail = deMail },
            Elster elster => new ReplyChannel() { Elster = elster },
            EMail eMail => new ReplyChannel() { EMail = eMail },
            Fink fink => new ReplyChannel() { Fink = fink },
            _ => throw new ArgumentOutOfRangeException(nameof(replyChannel))
        };

    /// <summary>
    /// Creating an email reply channel
    /// </summary>
    /// <param name="address">mail address</param>
    /// <param name="pgp">PGP key if it has to be encrypted</param>
    /// <returns></returns>
    public static IReplyChannelType FromEMail(string address, string? pgp = null) =>
        new EMail(address) { PgpPublicKey = pgp };

    /// <summary>
    /// Creating a DeMail reply channel
    /// </summary>
    /// <param name="address">The DeMail address</param>
    /// <returns></returns>
    public static IReplyChannelType FromDeMail(string address) =>
        new DeMail(address);

    /// <summary>
    /// Creating an Elster reply channel
    /// </summary>
    /// <param name="accountId">The Account ID of the Elster account</param>
    /// <param name="geschaeftszeichen"></param>
    /// <param name="lieferTicket"></param>
    /// <returns></returns>
    public static IReplyChannelType FromElster(string accountId, string? geschaeftszeichen = null,
        string? lieferTicket = null) =>
        new Elster(accountId) {
            Geschaeftszeichen = geschaeftszeichen,
            LieferTicket = lieferTicket
        };

    /// <summary>
    /// Creating a Fink reply channel
    /// </summary>
    /// <param name="postBoxRef"></param>
    /// <param name="host"></param>
    /// <returns></returns>
    public static IReplyChannelType FromFink(string postBoxRef, Uri? host = null) =>
        new Fink(postBoxRef) {
            Host = host,
        };
}

/// <summary>
/// Akkreditierte Anbieter siehe
/// https://www.bsi.bund.de/DE/Themen/Oeffentliche-Verwaltung/Moderner-Staat/De-Mail/Akkreditierte-DMDA/akkreditierte-dmda_node.html
/// </summary>
public class DeMail : IReplyChannelType {
    internal DeMail() {
        Address = string.Empty;
    }

    public DeMail(string address) {
        Address = address;
    }

    [JsonProperty("address")]
    public string Address { get; set; }
}

public class EMail : IReplyChannelType {
    internal EMail() {
        Address = string.Empty;
    }

    public EMail(string address) {
        Address = address;
    }

    [JsonProperty("address")]
    public string Address { get; set; }

    /// <summary>
    /// Hilfe zur Erstellung gibt es in der Dokumentation unter
    /// https://docs.fitko.de/fit-connect/details/pgp-export
    /// </summary>
    [JsonProperty("pgpPublicKey", NullValueHandling = NullValueHandling.Ignore)]
    public string? PgpPublicKey { get; set; }
}

/// <summary>
/// Siehe https://www.elster.de/elsterweb/infoseite/elstertransfer_hilfe_schnittstellen
/// </summary>
public class Elster : IReplyChannelType {
    internal Elster() {
        AccountId = string.Empty;
    }

    public Elster(string accountId) {
        AccountId = accountId;
    }

    [JsonProperty("accountId")]
    public string AccountId { get; set; }

    [JsonProperty("geschaeftszeichen", NullValueHandling = NullValueHandling.Ignore)]
    [JsonConverter(typeof(TentacledMinMaxLengthCheckConverter))]
    public string? Geschaeftszeichen { get; set; }

    [JsonProperty("lieferTicket", NullValueHandling = NullValueHandling.Ignore)]
    public string? LieferTicket { get; set; }
}

/// <summary>
/// Postfachadresse in einem interoperablen Servicekonto (FINK.PFISK)
/// </summary>
public class Fink : IReplyChannelType {
    internal Fink() {
        FinkPostfachRef = string.Empty;
    }

    public Fink(string finkPostfachRef) {
        FinkPostfachRef = finkPostfachRef;
    }

    /// <summary>
    /// FINK Postfachadresse
    /// </summary>
    [JsonProperty("finkPostfachRef")]
    [JsonConverter(typeof(StickyMinMaxLengthCheckConverter))]
    public string FinkPostfachRef { get; set; }

    /// <summary>
    /// URL des Servicekontos, in dem das Ziel-Postfach liegt
    /// </summary>
    [JsonProperty("host", NullValueHandling = NullValueHandling.Ignore)]
    public Uri? Host { get; set; }
}
